/*  $Id: WeakHashtable.java 1035 2010-06-15 13:08:33Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2002-2012 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.xml;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.IOException;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import net.eduvax.util.App;
import net.eduvax.util.ErrorHandler;
import net.eduvax.xml.SaxParser;
import net.eduvax.xml.SwitchableSaxHandler;

/** 
 * Services communs de désérialisation xml.
 */
public class XmlReader implements Runnable {
    /** 
     * Gestionnaire d'erreur.
     * @djin.relation aggregation
     */
    private ErrorHandler _errHandler;
    /** Gestionnaire lecture element racine
     * @djin.relation aggregation
     */ 
    private SwitchableSaxHandler _rootHandler;
    /** Nom de la source à lire 
     * <p>Nom du fichier si c'est un fichier...</p>
     */ 
    private String _sourceName;
    /** Flux source. 
     * @djin.relation composition
     */ 
    private InputStream _sourceStream;
    /** 
     * Indiquateur de la fermeture automatique du flux quand
     * la fermeture de l'entité racine à été lue. 
     */
    private boolean _autoClose = true;
    /** Activation de la fermeture implicite du flux en fin de 
     * lecture./
     * @param autoClose 
     * - true : le flux sera automatiquement fermé quand la fermeture de
     * la balise racine est rencontrée.
     * - false: le flux est laissé ouvert en l'état après reception balise
     * de fin.
     */ 
    public void setAutoClose(boolean autoClose) {
        _autoClose=autoClose;
    }

    /** 
     * Création d'un nouveau lecteur XML
     */	
    protected XmlReader() {
        _errHandler=App.get().getErrHandler();
    }

    /** 
     * Création d'un nouveau lecteur XML
     *	@param autoClose activation fermeture automatique :
     * - true : fermeture automatique du flux de lecture sous jacent
     * quand la fermeture de l'element racine est reçue.
     * - false : pas de fermeture (ce sera alors à la charge de 
     *	l'appelant)
     */	
    protected XmlReader(boolean autoClose) {
        this();
        _autoClose=autoClose;
        _errHandler=App.get().getErrHandler();
    }

    /** 
     * Attribuer un gestionnaire d'erreur.
     * @param errHandler Gestionnaire d'erreur
     */ 
    public void setErrHandler(ErrorHandler errHandler) {
        _errHandler=errHandler;
    }
    /** 
     * Préciser au lecteur le chemin du fichier à lire.
     * @param source chemin du fichier.
     */ 
    public void setSource(String source) {
        _sourceName=source;
        try {
            _sourceStream=new FileInputStream(source);
        }
        catch (FileNotFoundException ex) {
            Error err=new Error(""+getClass()+".nofile", Error.ELevel.ERR);
            err.addArg(_sourceName);
            handleErr(err);
        }
    }
    /** 
     * Préciser au lecteur le flux à lire.
     * @param source flux source des données XML.
     */ 
    public void setSource(InputStream source) {
        _sourceName="stream:"+source;
        _sourceStream=source;
    }
    /** Definition de l'élément racine.
     * Décrire l'élément racine attendu : son nom et comment le lire.
     * @param handler Gestionnaire lecture élément racine
     */ 
    public void setRootHandler(DefaultHandler handler) {
        _rootHandler=new SwitchableSaxHandler();
        _rootHandler.setHandler(handler);
    }
    public void setHandler(DefaultHandler handler) {
        if (_rootHandler!=null) {
            _rootHandler.setHandler(handler);
        }
        else {
            Error err=new Error(""+getClass()+".root",Error.ELevel.ERR);
            err.addArg(handler);
            handleErr(err);
        }
    }

    /** 
 * TODO plus nécessaire ?
     * Obtenir le reader XML sous-jacent.
     * <para>Attention, tant que setSource n'a pas été invoqué, il
     * n'y a pas de reader sous-jacent donc null est retourné.</para>
     * @return Le reader si initialisé, null sinon. 
    public XmlTextReader getXmlTextReader()
    {
        return _reader;
    }	
*/
    /** 
     * Declancher la lecture à partir de l'élément racine
     */
    public void run() {
        App.get().log().info("Opening for XML reading: "+_sourceName);
        Error err=null;
        try {
            SaxParser parser=SaxParser.getInputStreamParser(
                                            _sourceStream,_rootHandler);
            parser.parse();
        }
        catch (SAXException saxex) {
            err=new Error(""+getClass()+".sax", Error.ELevel.ERR);
        }
        catch (IOException ioex) {
            err=new Error(""+getClass()+".io", Error.ELevel.ERR);
        }
        if (err!=null) {
            err.addArg(_sourceName);
            handleErr(err);
        }
        if (_autoClose&&_sourceStream!=null) {
            try {
                _sourceStream.close();
            }
            catch (IOException ex) {
                err=new Error(""+getClass()+".close", Error.ELevel.ERR);
                err.addArg(_sourceName);
                handleErr(err);
            }
        }	
    }
    /** Prise en charge d'une erreur
     * <p>L'erreur est confiée au gestionnaire d'erreur mais au préalable
     * deux arguments lui sont ajoutés :</p>
     * - nom de la source (nom du fichier)
     * - n° de ligne courant (donc normalement n° de la ligne où l'erreur
     * est détéctée.
     * @param err Erreur à traiter
     */ 
    public void handleErr(net.eduvax.util.Error err) {
        err.addArg(_sourceName);
        // TODO err.addArg(_reader.LineNumber);
        _errHandler.handle(err);
    }
}
